var qwickup = {
	lg : Components.classes["@mozilla.org/consoleservice;1"]
			.getService(Components.interfaces.nsIConsoleService),
	pf : Components.classes["@mozilla.org/preferences-service;1"]
			.getService(Components.interfaces.nsIPrefService),
	wm : Components.classes["@mozilla.org/appshell/window-mediator;1"]
			.getService(Components.interfaces.nsIWindowMediator),
	lookup_url : 'http://qwickup.com/',
	browser : 'chrome://qwickup/content/browser.xul',
	popupspec : "chrome,centerscreen,status,scrollbars,resizable",
	win : content,
	doc : document,
	log : function(msg) {
		this.lg.logStringMessage("Qwickup X: " + msg);
	},
	onLoad : function() {
		this.pref = this.pf.getBranch("extensions.qwickup.");
		this.firstrun();
		var contextMenu = document.getElementById("contentAreaContextMenu");
		if (contextMenu) {
			contextMenu.addEventListener("popupshowing", function(e) {
				qwickup.onContextMenuShowing(e);
			}, false);
		}
		if (gBrowser) {
			gBrowser.addEventListener("DOMContentLoaded", function(e) {
				new QwickUpContent(e.originalTarget,
						e.originalTarget.defaultView);
			}, false);
		}
	},
	onContextMenuShowing : function(event) {
		document.getElementById("qwickup-contextmenu").hidden = !qwickup.contextEnabled();
		document.getElementById("qwickup-contextmenu-sep").hidden = !qwickup.contextEnabled();
	},
	onToolbarMenuShowing : function(menu) {
		var inlineEnable = document.getElementById("toolbarMenu-enableInline");
		inlineEnable.setAttribute('checked', this.inlineEnabled());
		var contextEnable = document
				.getElementById("toolbarMenu-enableContext");
		contextEnable.setAttribute('checked', this.contextEnabled());
		var modes = [ "tab", "pintab", "popup" ];
		var lastMode = this.getLastMode();
		for ( var mode in modes) {
			mode = modes[mode];
			var el = document.getElementById("qwickup-openin" + mode);
			if (el) {
				el.style.fontWeight = (lastMode === mode ? 'bold' : '');
			}
		}
	},
	inlineEnabled : function() {
		return this.isEnabled("enableInline", false);
	},
	contextEnabled : function() {
		return this.isEnabled("enableContext", true);
	},
	isEnabled : function(name, defaultValue) {
		try {
			return this.pref.getBoolPref(name);
		} catch (e) {
		}
		return defaultValue;
	},
	getLastMode : function() {
		var mode = null;
		try {
			mode = this.pref.getCharPref("lastMode");
		} catch (e) {
		}
		return mode || "tab";
	},
	inlineCheckbox : function(event) {
		this.checkboxed(event, "enableInline");
	},
	contextCheckbox : function(event) {
		this.checkboxed(event, "enableContext");
	},
	checkboxed : function(event, name) {
		if (event.target.getAttribute('checked')) {
			this.pref.setBoolPref(name, true);
		} else {
			this.pref.setBoolPref(name, false);
		}
	},
	lookup : function(e, txt, mode) {
		if (!e) {
			return;
		}
		txt = txt || this.getSelection();
		// var newurl = this.lookup_url + '#' + txt;

		// See: http://mxr.mozilla.org/mozilla/ident?i=whereToOpenLink
		var shift = e.shiftKey;
		var ctrl = e.ctrlKey || e.metaKey;
		var middle = e.button == 1;

		// if any key used...
		if (ctrl || middle) {
			// on ctrl+shift set focus on new tab
			mode = 'tab';
		} else if (shift) {
			mode = 'popup';
		}

		// if any mode explicitly set, by key of from menu...
		if (mode === 'tab' || mode === 'popup' || mode === 'pintab') {
			this.pref.setCharPref("lastMode", mode);

		} else if (this.isQwickup(content.location)
				|| this.isBlank(content.location)) {
			// else check for current location
			mode = 'current';

		} else {
			// else get last mode
			mode = this.getLastMode("lastMode");
		}

		if (mode === 'tab' || mode === 'pintab') {
			if (this.isQwickup(content.location) || !this.openOldTab(txt)) {
				this.openNewTab(txt, mode === 'pintab');
			}

		} else if (mode === 'current') {
			this.openHere(txt);
			return;

		} else if (mode === 'popup' || true) {
			// popup or any other unknown mode
			if (!this.openOldPopup(txt)) {
				this.openNewPopup(txt);
			}
		}
	},

	isQwickup : function(url) {
		if (!url || !(url = url + "")) {
			return false;
		}
		return url === this.lookup_url
				|| this.startswith(url, this.lookup_url + "?")
				|| this.startswith(url, this.lookup_url + "#");
	},

	isBlank : function(url) {
		return url && this.startswith(url, "about:");
	},

	startswith : function(str, prefix) {
		return (str + "").substring(0, prefix.length) === prefix;
	},

	isJSLocation : function(url) {
		var is = url && this.startswith((url + ""), "javascript:");
		return is;
	},

	makeUrl : function(win, txt) {
		txt = txt || "";
		var url;
		if (win && this.isQwickup(url = win.location + "")) {
			var split = url.split("#", 2);
			url = split[0] + "#";
			if (split.length > 1) {
				split = split[1].split("/", 2);
				if (split.length > 1) {
					url += split[0] + "/";
				}
			}
			url += txt;
		} else {
			url = this.lookup_url + '?ap=ff'
					+ (txt ? "&query=" + encodeURIComponent(txt) : "");
		}
		if (win) {
			win.location = url;
		}
		return url;
	},

	openHere : function(txt) {
		this.makeUrl(content.window, txt);
	},

	openOldPopup : function(txt) {
		var currentWindow = this.wm.getMostRecentWindow("navigator:browser");
		var wins = this.wm.getEnumerator("navigator:browser");
		while (!popup && wins.hasMoreElements()) {
			var win = wins.getNext();
			if (win.__qwickup_popup && !win.__qwickup_popup.closed) {
				var popup = win.__qwickup_popup;
				this.makeUrl(popup.content.window, txt);
				popup.focus();
				return true;
			}
		}
		return false;
	},

	openNewPopup : function(txt) {
		var currentWindow = this.wm.getMostRecentWindow("navigator:browser");
		var popup = currentWindow.open(this.browser, 'QwickUp', this.popupspec
				+ (this.inlineEnabled() ? "" : ",toolbar"));
		currentWindow.__qwickup_popup = popup;
		var self = this;
		popup.addEventListener("load", function() {
			self.makeUrl(popup.content.window, txt);
			popup.focus();
		}, false);
	},

	openOldTab : function(txt) {
		var currentWindow = this.wm.getMostRecentWindow("navigator:browser");
		var gBrowser = currentWindow.gBrowser;
		var numTabs = gBrowser.browsers.length;
		for ( var i = 0; i < numTabs; i++) {
			var tab = gBrowser.getBrowserAtIndex(i);
			if (this.isQwickup(tab.contentWindow.location)) {
				this.makeUrl(tab.contentWindow, txt);
				gBrowser.selectedTab = gBrowser.tabContainer.childNodes[i];
				currentWindow.focus();
				return true;
			}
		}
		return false;
	},

	openNewTab : function(txt, pin) {
		var currentWindow = this.wm.getMostRecentWindow("navigator:browser");
		var url = this.makeUrl(null, txt);
		if (currentWindow) {
			var gBrowser = currentWindow.gBrowser;
			var tab = (gBrowser.loadOneTab ? gBrowser.loadOneTab(url)
					: gBrowser.addTab(url));
			gBrowser.selectedTab = tab;
			if (pin) {
				gBrowser.pinTab(tab);
			}
			currentWindow.focus();
		} else {
			window.open(url).focus();
		}
	},

	getSelection : function() {
		var selection = null;
		try {
			var el = document.commandDispatcher.focusedElement;

			if (el && el.tagName) {
				var tag = el.tagName.toUpperCase();
				if (tag == "TEXTAREA" || tag == "INPUT") {
					selection = el.value.substr(el.selectionStart,
							el.selectionEnd - el.selectionStart);
				}
			}
		} catch (e) {
		}

		try {
			if (!selection) {
				// content.getSelection();
				selection = (document.commandDispatcher.focusedWindow
						.getSelection() || "").toString();
			}
		} catch (e) {
		}

		return (selection || "").replace(/^\s\s*/, "").replace(/\s\s*$/, "");
	},

	firstrun : function() {
		var firstrun = true;
		try {
			firstrun = this.pref.getBoolPref("firstrun");
		} catch (e) {
		}
		if (firstrun) {
			this.pref.setBoolPref("firstrun", false);
			this.addButton();
			// window.setTimeout(function() {
			// gBrowser.selectedTab = gBrowser.addTab("about:mozilla");
			// }, 1500);
		}

	},

	addButton : function() {
		var myid = "qwickup-toolbar-button";
		var navbar = document.getElementById("nav-bar");
		if (!navbar) {
			return;
		}

		var current = navbar.currentSet;
		if (current.indexOf(myid) != -1) {
			return;
		}

		var other;
		if (current.indexOf(other = "urlbar-container") != -1) {
			current = current.replace(other, myid + "," + other);

		} else if (current.indexOf(other = "home-button") != -1) {
			current = current.replace(other, other + "," + myid);

		} else if (current.indexOf(other = "stop-button") != -1) {
			current = current.replace(other, other + "," + myid);

		} else if (current.indexOf(other = "reload-button") != -1) {
			current = current.replace(other, other + "," + myid);

		} else if (current.indexOf(other = "unified-back-forward-button") != -1) {
			current = current.replace(other, other + "," + myid);

		} else {
			current = myid + "," + current;
		}

		navbar.setAttribute("currentset", current);
		navbar.currentSet = current;
		document.persist(navbar.id, "currentset");
		try {
			BrowserToolboxCustomizeDone(true);
		} catch (e) {
		}
	}
};

function QwickUpContent(document, window) {
	var me = this;

	this.init = function() {
		// if (window != window.top || window.frameElement) return;
		if (document.nodeName != "#document"
				|| qwickup.isBlank(document.location)
				|| qwickup.isJSLocation(document.location)) {
			return;
		}

		this.injectStyles(".qwickup_inline_button{display:none;}");
		this.injectStyles(".qwickup_hide_on_ext{display:none;}");

		document.addEventListener("mouseup", function(e) {
			me.onmouseup(e);
		}, false);
		document.addEventListener("keyup", function(e) {
			me.onkey(e);
		}, false);
		document.addEventListener("keydown", function(e) {
			me.onkey(e);
		}, false);
	};
	this.insert = function() {
		if (this.b) {
			return;
		}
		var b = document.createElement("span");
		this.clean(b);
		b.className = "qwickup_outline_button";
		b.style.display = "none";
		b.style.position = 'absolute';
		b.style.zIndex = 99999;
		document.body.appendChild(b);

		var a = document.createElement("a");
		this.clean(a);
		a.href = '#';
		// a.style.cursor= "pointer";
		a.title = 'QwickUp';
		a.addEventListener("mousedown", function(e) {
			me.onmousedown(e);
		}, false);
		a.addEventListener("click", function(e) {
			me.onclick(e);
		}, false);
		b.appendChild(a);

		var img = document.createElement("img");
		this.clean(img);
		img.src = "resource://qwickup/skin/q16.png";
		a.appendChild(img);

		this.b = b;
	};
	this.remove = function() {
		if (!this.b) {
			return;
		}
		this.b.parentNode.removeChild(this.b);
		this.b = null;
	};
	// icon
	this.onmousedown = function(e) {
		// qwickup.log("onmousedown");
		this.push(e);
	};
	// icon
	this.onclick = function(e) {
		// qwickup.log("onclick");
		e.preventDefault();
		qwickup.lookup(e, me.selection);
		me.hide();
	};
	// doc
	this.onkey = function(e) {
		// qwickup.log("onkey");
		if ((e && (e.altKey || e.keyCode == 27)) || !qwickup.getSelection()) {
			this.hide();
			return;
		}
	};
	// doc
	this.onmouseup = function(e) {
		if (!qwickup.inlineEnabled()) {
			return;
		}

		// qwickup.log("onmouseup " + me.selection + "/" +
		// qwickup.getSelection());

		// Sometime clicking on icon makes getSelection() to return null
		// but removing icon here avoids onclick to happened. For example if
		// selection is inside input and input loses its focus as a result
		// of clicking on icon.
		if (this.peek()) {
			// qwickup.log('>hidetimeout');
			this.hidetimeout(100);
			return;
		}

		if (!qwickup.getSelection()) {
			this.hide();
			return;
		}
		var xy = this.getSelectionXY(e);
		if (xy) {
			this.show(xy.x, xy.y);
		}
	};

	this.getSelectionXY = function(e) {
		try {
			var el = qwickup.doc.commandDispatcher.focusedElement;
			if (el && el.tagName) {
				var tag = el.tagName.toUpperCase();
				if (tag == "TEXTAREA" || tag == "INPUT") {
					if (e && (e.pageX || e.pageY)) {
						return {
							x : e.pageX + 10,
							y : e.pageY + 10
						};
					}
					return null;
				}
			}
		} catch (e) {
		}

		var selection = qwickup.doc.commandDispatcher.focusedWindow
				.getSelection();
		if (selection) {
			try {
				var range = selection.getRangeAt(0).cloneRange();
				range.collapse(false);

				var marker = document.createElement("span");
				marker.appendChild(document.createTextNode("\ufeff"));
				range.insertNode(marker);

				var xy = this.offset(marker);
				xy.y += marker.offsetHeight;
				marker.parentNode.removeChild(marker);
				return xy;
			} catch (e) {
			}
		}
		return null;
	};
	this.offset = function(el) {
		var rect = el.getBoundingClientRect();
		return {
			x : rect.left + window.pageXOffset,
			y : rect.top + window.pageYOffset
		};
	};

	this.show = function(x, y) {
		this.remove_it = false;
		this.insert();
		window.clearTimeout(this.timeout);
		this.b.style.display = "";
		this.b.style.top = (y) + "px";
		this.b.style.left = (x) + "px";
		this.timeout = window.setTimeout(function() {
			me.hide();
		}, 4000);
		window.setTimeout(function() {
			me.onkey();
		}, 100);
	};
	this.hidetimeout = function(timeout, callback) {
		this.remove_it = true;
		return window.setTimeout(function() {
			if (me.remove_it) {
				me.hide();
			}
			if (callback) {
				callback(me.remove_it);
			}
		}, timeout);
	};
	this.hide = function() {
		// see onmouseup
		this.pop();

		if (!this.b) {
			return;
		}
		this.b.style.display = "none";
		this.remove();
	};

	this.push = function(e) {
		return (me.selection = qwickup.getSelection(e));
	};
	this.pop = function() {
		var txt = me.selection;
		me.selection = null;
		return txt;
	};
	this.peek = function() {
		var txt = me.selection;
		return txt;
	};

	this.injectStyles = function(rules) {
		rules = document.createTextNode(rules);
		var style = document.createElement("style");
		style.type = "text/css";
		if (style.styleSheet) {
			style.styleSheet.cssText = rules.nodeValue;
		} else {
			style.appendChild(rules);
		}
		var head = document.getElementsByTagName("head")[0];
		if (head) {
			head.appendChild(style);
		} else {
			var body = document.getElementsByTagName("body")[0];
			if (body) {
				body.appendChild(style);
			} else {
				return false;
			}
		}
		return true;
	};
	this.clean = function(el) {
		el.style.position = "relative";
		el.style.border = "0 none";
		el.style.padding = "0";
		el.style.margin = "0";
		el.style.background = "none trasparent";
		el.style.verticalAlign = "baseline";
		el.style.lineHeight = "1";
	};
	this.init();
}

window.addEventListener("load", function() {
	qwickup.onLoad();
}, false);